import warnings


LEGACY_CONFIG_OPTIONS = {
    "REDIS_URL": "RQ_DASHBOARD_REDIS_URL",
    "REDIS_HOST": "RQ_DASHBOARD_REDIS_HOST",
    "REDIS_PORT": "RQ_DASHBOARD_REDIS_PORT",
    "REDIS_PASSWORD": "RQ_DASHBOARD_REDIS_PASSWORD",
    "REDIS_DB": "RQ_DASHBOARD_REDIS_DB",
    "REDIS_SENTINELS": "RQ_DASHBOARD_REDIS_SENTINELS",
    "REDIS_MASTER_NAME": "RQ_DASHBOARD_REDIS_MASTER_NAME",
    "RQ_POLL_INTERVAL": "RQ_DASHBOARD_POLL_INTERVAL",
    "WEB_BACKGROUND": "RQ_DASHBOARD_WEB_BACKGROUND",
    "DELETE_JOBS": "RQ_DASHBOARD_DELETE_JOBS",
}

warning_template = (
    "Configuration option {old_name} is deprecated and will be removed in future versions. "
    "Please use {new_name} instead."
)


def upgrade_config(app):
    """
    Updates old configuration options with new ones throwing warnings to those who haven't upgraded yet.
    """
    for old_name, new_name in LEGACY_CONFIG_OPTIONS.items():
        if old_name in app.config and new_name not in app.config:
            warnings.warn(
                warning_template.format(old_name=old_name, new_name=new_name),
                UserWarning,
            )
            app.config[new_name] = app.config[old_name]



import json
import unittest

import redis
from rq import Queue, Worker, pop_connection, push_connection

from rq_dashboard.cli import make_flask_app


HTTP_OK = 200

class BasicTestCase(unittest.TestCase):
    redis_client = None

    def get_redis_client(self):
        if self.redis_client is None:
            self.redis_client = redis.Redis()
        return self.redis_client

    def setUp(self):
        self.app = make_flask_app(None, None, None, '')
        self.app.testing = True
        self.app.config['RQ_DASHBOARD_REDIS_URL'] = 'redis://127.0.0.1'
        self.app.redis_conn = self.get_redis_client()
        push_connection(self.get_redis_client())
        self.client = self.app.test_client()

    def tearDown(self):
        pop_connection()

    def test_dashboard_ok(self):
        response = self.client.get('/')
        self.assertEqual(response.status_code, HTTP_OK)

    def test_queues_list_json(self):
        response = self.client.get('/0/data/queues.json')
        self.assertEqual(response.status_code, HTTP_OK)
        data = json.loads(response.data.decode('utf8'))
        self.assertIsInstance(data, dict)
        self.assertIn('queues', data)
        self.assertEqual(response.headers['Cache-Control'], 'no-store')

    def test_workers_list_json(self):
        response = self.client.get('/0/data/workers.json')
        self.assertEqual(response.status_code, HTTP_OK)
        data = json.loads(response.data.decode('utf8'))
        self.assertIsInstance(data, dict)
        self.assertIn('workers', data)

    def test_queued_jobs_list(self):
        response_dashboard = self.client.get('/0/view/jobs')
        self.assertEqual(response_dashboard.status_code, HTTP_OK)
        response_queued = self.client.get('/0/view/jobs/default/queued/8/1')
        self.assertEqual(response_queued.status_code, HTTP_OK)
        response = self.client.get('/0/data/jobs/default/queued/8/1.json')
        self.assertEqual(response.status_code, HTTP_OK)
        data = json.loads(response.data.decode('utf8'))
        self.assertIsInstance(data, dict)
        self.assertIn('jobs', data)

    def test_single_job_view(self):
        def some_work():
            return
        q = Queue(connection=self.app.redis_conn)
        job = q.enqueue(some_work)
        job_url = '/0/view/job/' + job.id
        response = self.client.get(job_url)
        self.assertEqual(response.status_code, HTTP_OK)
        job.delete()

    def test_del_job_mechanism(self):
        def some_work():
            return
        q = Queue(connection=self.app.redis_conn)
        job = q.enqueue(some_work)
        job_del_url = '/job/' + job.id + '/delete'
        response_del = self.client.post(job_del_url)
        self.assertEqual(response_del.status_code, HTTP_OK)

    def test_registry_jobs_list(self):
        response = self.client.get('/0/data/jobs/default/failed/8/1.json')
        self.assertEqual(response.status_code, HTTP_OK)
        data = json.loads(response.data.decode('utf8'))
        self.assertIsInstance(data, dict)
        self.assertIn('jobs', data)

    def test_worker_python_version_field(self):
        w = Worker(['q'])
        w.register_birth()
        response = self.client.get('/0/data/workers.json')
        data = json.loads(response.data.decode('utf8'))
        if getattr(w, 'python_version', None):
            self.assertEqual(w.python_version, data['workers'][0]['python_version'])
        else:
            self.assertEqual('', data['workers'][0]['python_version'])
        w.register_death()

    def test_worker_version_field(self):
        w = Worker(['q'])
        w.register_birth()
        response = self.client.get('/0/data/workers.json')
        data = json.loads(response.data.decode('utf8'))
        if getattr(w, 'version', None):
            self.assertEqual(w.version, data['workers'][0]['version'])
        else:
            self.assertEqual('', data['workers'][0]['version'])
        w.register_death()


__all__ = [
    'BasicTestCase',
]



import os

from setuptools import find_packages, setup


def get_version():
    basedir = os.path.dirname(__file__)
    with open(os.path.join(basedir, 'rq_dashboard/version.py')) as f:
        locals = {}
        exec(f.read(), locals)
        return locals['VERSION']
    raise RuntimeError('No version info found.')


with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name='rq-dashboard',
    version=get_version(),
    url='https://github.com/Parallels/rq-dashboard',
    license='BSD',
    author='Vincent Driessen',
    author_email='vincent@3rdcloud.com',
    description='rq-dashboard is a general purpose, lightweight, web interface'
                ' to monitor your RQ queues, jobs, and workers in realtime.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(exclude=('tests',)),
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=['rq>=1.0', 'Flask', 'redis', 'arrow', 'redis-sentinel-url'],
    entry_points={
        'console_scripts': [
            'rq-dashboard = rq_dashboard.cli:main'
        ]
    },
    classifiers=[
        # As from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        # 'Development Status :: 1 - Planning',
        # 'Development Status :: 2 - Pre-Alpha',
        # 'Development Status :: 3 - Alpha',
        'Development Status :: 4 - Beta',
        # 'Development Status :: 5 - Production/Stable',
        # 'Development Status :: 6 - Mature',
        # 'Development Status :: 7 - Inactive',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX',
        'Operating System :: MacOS',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Internet',
        'Topic :: Scientific/Engineering',
        'Topic :: System :: Distributed Computing',
        'Topic :: System :: Systems Administration',
        'Topic :: System :: Monitoring',
    ]
)



"""RQ Dashboard Flask Blueprint.

Uses the standard Flask configuration mechanism e.g. to set the connection
parameters to REDIS. To keep the documentation and defaults all in once place
the default settings must be loaded from ``rq_dashboard.default_settings``
e.g.  as done in ``cli.py``.

RQ Dashboard does not contain any built-in authentication mechanism because

    1. it is the responsbility of the wider hosting app rather than a
       particular blueprint, and

    2. there are numerous ways of adding security orthogonally.

As a quick-and-dirty convenience, the command line invocation in ``cli.py``
provides the option to require HTTP Basic Auth in a few lines of code.

"""
import os
import re
from functools import wraps
from math import ceil

import arrow
from flask import (
    Blueprint,
    current_app,
    make_response,
    render_template,
    request,
    send_from_directory,
    url_for,
)
from redis.exceptions import ConnectionError as RedisConnectionError
from redis_sentinel_url import connect as from_url
from rq import (
    VERSION as rq_version,
    Queue,
    Worker,
    pop_connection,
    push_connection,
    requeue_job,
)
from rq.job import Job
from rq.registry import (
    DeferredJobRegistry,
    FailedJobRegistry,
    FinishedJobRegistry,
    StartedJobRegistry,
)
from six import string_types

from .legacy_config import upgrade_config
from .version import VERSION as rq_dashboard_version


blueprint = Blueprint(
    "rq_dashboard", __name__, template_folder="templates", static_folder="static",
)


# @blueprint.before_app_first_request
def setup_rq_connection(current_app):
    # we need to do It here instead of cli, since It may be embeded
    upgrade_config(current_app)
    # Getting Redis connection parameters for RQ
    redis_url = current_app.config.get("RQ_DASHBOARD_REDIS_URL")
    if isinstance(redis_url, string_types):
        current_app.config["RQ_DASHBOARD_REDIS_URL"] = (redis_url,)
        _, current_app.redis_conn = from_url((redis_url,)[0])
    elif isinstance(redis_url, (tuple, list)):
        _, current_app.redis_conn = from_url(redis_url[0])
    else:
        raise RuntimeError("No Redis configuration!")


@blueprint.before_request
def push_rq_connection():
    new_instance_number = request.view_args.get("instance_number")
    if new_instance_number is not None:
        redis_url = current_app.config.get("RQ_DASHBOARD_REDIS_URL")
        if new_instance_number < len(redis_url):
            _, new_instance = from_url(redis_url[new_instance_number])
        else:
            raise LookupError("Index exceeds RQ list. Not Permitted.")
    else:
        new_instance = current_app.redis_conn
    push_connection(new_instance)
    current_app.redis_conn = new_instance


@blueprint.teardown_request
def pop_rq_connection(exception=None):
    pop_connection()


def jsonify(f):
    @wraps(f)
    def _wrapped(*args, **kwargs):
        from flask import jsonify as flask_jsonify

        result_dict = f(*args, **kwargs)
        return flask_jsonify(**result_dict), {"Cache-Control": "no-store"}

    return _wrapped


def serialize_queues(instance_number, queues):
    return [
        dict(
            name=q.name,
            count=q.count,
            queued_url=url_for(
                ".jobs_overview",
                instance_number=instance_number,
                queue_name=q.name,
                registry_name="queued",
                per_page="8",
                page="1",
            ),
            failed_job_registry_count=FailedJobRegistry(q.name).count,
            failed_url=url_for(
                ".jobs_overview",
                instance_number=instance_number,
                queue_name=q.name,
                registry_name="failed",
                per_page="8",
                page="1",
            ),
            started_job_registry_count=StartedJobRegistry(q.name).count,
            started_url=url_for(
                ".jobs_overview",
                instance_number=instance_number,
                queue_name=q.name,
                registry_name="started",
                per_page="8",
                page="1",
            ),
            deferred_job_registry_count=DeferredJobRegistry(q.name).count,
            deferred_url=url_for(
                ".jobs_overview",
                instance_number=instance_number,
                queue_name=q.name,
                registry_name="deferred",
                per_page="8",
                page="1",
            ),
            finished_job_registry_count=FinishedJobRegistry(q.name).count,
            finished_url=url_for(
                ".jobs_overview",
                instance_number=instance_number,
                queue_name=q.name,
                registry_name="finished",
                per_page="8",
                page="1",
            ),
        )
        for q in queues
    ]


def serialize_date(dt):
    if dt is None:
        return None
    return arrow.get(dt).to("UTC").datetime.isoformat()


def serialize_job(job):
    return dict(
        id=job.id,
        created_at=serialize_date(job.created_at),
        ended_at=serialize_date(job.ended_at),
        exc_info=str(job.exc_info) if job.exc_info else None,
        description=job.description,
    )


def serialize_current_job(job):
    if job is None:
        return "idle"
    return dict(
        job_id=job.id,
        description=job.description,
        created_at=serialize_date(job.created_at),
        call_string=job.get_call_string(),
    )


def remove_none_values(input_dict):
    return dict(((k, v) for k, v in input_dict.items() if v is not None))


def pagination_window(total_items, cur_page, per_page, window_size=10):
    all_pages = range(1, int(ceil(total_items / float(per_page))) + 1)
    result = all_pages
    if window_size >= 1:
        temp = min(
            len(all_pages) - window_size, (cur_page - 1) - int(ceil(window_size / 2.0))
        )
        pages_window_start = max(0, temp)
        pages_window_end = pages_window_start + window_size
        result = all_pages[pages_window_start:pages_window_end]
    return result


@blueprint.route("/favicon.ico")
def favicon():
    return send_from_directory(
        os.path.join(blueprint.root_path, "static"),
        "favicon.ico",
        mimetype="image/vnd.microsoft.icon",
    )


def get_queue_registry_jobs_count(queue_name, registry_name, offset, per_page):
    queue = Queue(queue_name)
    if registry_name != "queued":
        if per_page >= 0:
            per_page = offset + (per_page - 1)

        if registry_name == "failed":
            current_queue = FailedJobRegistry(queue_name)
        elif registry_name == "deferred":
            current_queue = DeferredJobRegistry(queue_name)
        elif registry_name == "started":
            current_queue = StartedJobRegistry(queue_name)
        elif registry_name == "finished":
            current_queue = FinishedJobRegistry(queue_name)
    else:
        current_queue = queue
    total_items = current_queue.count

    job_ids = current_queue.get_job_ids(offset, per_page)
    current_queue_jobs = [queue.fetch_job(job_id) for job_id in job_ids]
    jobs = [serialize_job(job) for job in current_queue_jobs]

    return (total_items, jobs)


def escape_format_instance_list(url_list):
    if isinstance(url_list, (list, tuple)):
        url_list = [re.sub(r"://:[^@]*@", "://:***@", x) for x in url_list]
    elif isinstance(url_list, string_types):
        url_list = [re.sub(r"://:[^@]*@", "://:***@", url_list)]
    return url_list


@blueprint.route("/", defaults={"instance_number": 0})
@blueprint.route("/<int:instance_number>/")
@blueprint.route("/<int:instance_number>/view")
@blueprint.route("/<int:instance_number>/view/queues")
def queues_overview(instance_number):
    try:
        r = make_response(
            render_template(
                "rq_dashboard/queues.html",
                current_instance=instance_number,
                instance_list=escape_format_instance_list(current_app.config.get("RQ_DASHBOARD_REDIS_URL")),
                queues=Queue.all(),
                rq_url_prefix=url_for(".queues_overview"),
                rq_dashboard_version=rq_dashboard_version,
                rq_version=rq_version,
                active_tab="queues",
                deprecation_options_usage=current_app.config.get(
                    "DEPRECATED_OPTIONS", False
                ),
            )
        )
    except RedisConnectionError:
        r = make_response(
            "<h1>Connection Error</h1>"
        )
    r.headers.set("Cache-Control", "no-store")
    return r


@blueprint.route("/<int:instance_number>/view/workers")
def workers_overview(instance_number):
    r = make_response(
        render_template(
            "rq_dashboard/workers.html",
            current_instance=instance_number,
            instance_list=escape_format_instance_list(current_app.config.get("RQ_DASHBOARD_REDIS_URL")),
            workers=Worker.all(),
            rq_url_prefix=url_for(".queues_overview"),
            rq_dashboard_version=rq_dashboard_version,
            rq_version=rq_version,
            active_tab="workers",
            deprecation_options_usage=current_app.config.get(
                "DEPRECATED_OPTIONS", False
            ),
        )
    )
    r.headers.set("Cache-Control", "no-store")
    return r


@blueprint.route(
    "/<int:instance_number>/view/jobs",
    defaults={
        "queue_name": None,
        "registry_name": "queued",
        "per_page": "8",
        "page": "1",
    },
)
@blueprint.route(
    "/<int:instance_number>/view/jobs/<queue_name>/<registry_name>/<int:per_page>/<int:page>"
)
def jobs_overview(instance_number, queue_name, registry_name, per_page, page):
    if queue_name is None:
        queue = Queue()
    else:
        queue = Queue(queue_name)
    r = make_response(
        render_template(
            "rq_dashboard/jobs.html",
            current_instance=instance_number,
            instance_list=escape_format_instance_list(current_app.config.get("RQ_DASHBOARD_REDIS_URL")),
            queues=Queue.all(),
            queue=queue,
            per_page=per_page,
            page=page,
            registry_name=registry_name,
            rq_url_prefix=url_for(".queues_overview"),
            rq_dashboard_version=rq_dashboard_version,
            rq_version=rq_version,
            active_tab="jobs",
            deprecation_options_usage=current_app.config.get(
                "DEPRECATED_OPTIONS", False
            ),
        )
    )
    r.headers.set("Cache-Control", "no-store")
    return r


@blueprint.route("/<int:instance_number>/view/job/<job_id>")
def job_view(instance_number, job_id):
    job = Job.fetch(job_id)
    r = make_response(
        render_template(
            "rq_dashboard/job.html",
            current_instance=instance_number,
            instance_list=escape_format_instance_list(current_app.config.get("RQ_DASHBOARD_REDIS_URL")),
            id=job.id,
            rq_url_prefix=url_for(".queues_overview"),
            rq_dashboard_version=rq_dashboard_version,
            rq_version=rq_version,
            deprecation_options_usage=current_app.config.get(
                "DEPRECATED_OPTIONS", False
            ),
        )
    )
    r.headers.set("Cache-Control", "no-store")
    return r


@blueprint.route("/job/<job_id>/delete", methods=["POST"])
@jsonify
def delete_job_view(job_id):
    job = Job.fetch(job_id)
    job.delete()
    return dict(status="OK")


@blueprint.route("/job/<job_id>/requeue", methods=["POST"])
@jsonify
def requeue_job_view(job_id):
    requeue_job(job_id, connection=current_app.redis_conn)
    return dict(status="OK")


@blueprint.route("/requeue/<queue_name>", methods=["GET", "POST"])
@jsonify
def requeue_all(queue_name):
    fq = Queue(queue_name).failed_job_registry
    job_ids = fq.get_job_ids()
    count = len(job_ids)
    for job_id in job_ids:
        requeue_job(job_id, connection=current_app.redis_conn)
    return dict(status="OK", count=count)


@blueprint.route("/queue/<queue_name>/<registry_name>/empty", methods=["POST"])
@jsonify
def empty_queue(queue_name, registry_name):
    if registry_name == "queued":
        q = Queue(queue_name)
        q.empty()
    elif registry_name == "failed":
        ids = FailedJobRegistry(queue_name).get_job_ids()
        for id in ids:
            delete_job_view(id)
    elif registry_name == "deferred":
        ids = DeferredJobRegistry(queue_name).get_job_ids()
        for id in ids:
            delete_job_view(id)
    elif registry_name == "started":
        ids = StartedJobRegistry(queue_name).get_job_ids()
        for id in ids:
            delete_job_view(id)
    elif registry_name == "finished":
        ids = FinishedJobRegistry(queue_name).get_job_ids()
        for id in ids:
            delete_job_view(id)
    return dict(status="OK")


@blueprint.route("/queue/<queue_name>/compact", methods=["POST"])
@jsonify
def compact_queue(queue_name):
    q = Queue(queue_name)
    q.compact()
    return dict(status="OK")


@blueprint.route("/<int:instance_number>/data/queues.json")
@jsonify
def list_queues(instance_number):
    queues = serialize_queues(instance_number, sorted(Queue.all()))
    return dict(queues=queues)


@blueprint.route(
    "/<int:instance_number>/data/jobs/<queue_name>/<registry_name>/<per_page>/<page>.json"
)
@jsonify
def list_jobs(instance_number, queue_name, registry_name, per_page, page):
    current_page = int(page)
    per_page = int(per_page)
    offset = (current_page - 1) * per_page
    total_items, jobs = get_queue_registry_jobs_count(
        queue_name, registry_name, offset, per_page
    )

    pages_numbers_in_window = pagination_window(total_items, current_page, per_page)
    pages_in_window = [
        dict(
            number=p,
            url=url_for(
                ".jobs_overview",
                instance_number=instance_number,
                queue_name=queue_name,
                registry_name=registry_name,
                per_page=per_page,
                page=p,
            ),
        )
        for p in pages_numbers_in_window
    ]
    last_page = int(ceil(total_items / float(per_page)))

    prev_page = None
    if current_page > 1:
        prev_page = dict(
            url=url_for(
                ".jobs_overview",
                instance_number=instance_number,
                queue_name=queue_name,
                registry_name=registry_name,
                per_page=per_page,
                page=(current_page - 1),
            )
        )

    next_page = None
    if current_page < last_page:
        next_page = dict(
            url=url_for(
                ".jobs_overview",
                instance_number=instance_number,
                queue_name=queue_name,
                registry_name=registry_name,
                per_page=per_page,
                page=(current_page + 1),
            )
        )

    first_page_link = dict(
        url=url_for(
            ".jobs_overview",
            instance_number=instance_number,
            queue_name=queue_name,
            registry_name=registry_name,
            per_page=per_page,
            page=1,
        )
    )
    last_page_link = dict(
        url=url_for(
            ".jobs_overview",
            instance_number=instance_number,
            queue_name=queue_name,
            registry_name=registry_name,
            per_page=per_page,
            page=last_page,
        )
    )

    pagination = remove_none_values(
        dict(
            current_page=current_page,
            num_pages=last_page,
            pages_in_window=pages_in_window,
            next_page=next_page,
            prev_page=prev_page,
            first_page=first_page_link,
            last_page=last_page_link,
        )
    )

    return dict(
        name=queue_name, registry_name=registry_name, jobs=jobs, pagination=pagination
    )


@blueprint.route("/<int:instance_number>/data/job/<job_id>.json")
@jsonify
def job_info(instance_number, job_id):
    job = Job.fetch(job_id)
    return dict(
        id=job.id,
        created_at=serialize_date(job.created_at),
        enqueued_at=serialize_date(job.enqueued_at),
        ended_at=serialize_date(job.ended_at),
        origin=job.origin,
        status=job.get_status(),
        result=job._result,
        exc_info=str(job.exc_info) if job.exc_info else None,
        description=job.description,
    )


@blueprint.route("/<int:instance_number>/data/workers.json")
@jsonify
def list_workers(instance_number):
    def serialize_queue_names(worker):
        return [q.name for q in worker.queues]

    workers = sorted(
        (
            dict(
                name=worker.name,
                queues=serialize_queue_names(worker),
                state=str(worker.get_state()),
                current_job=serialize_current_job(worker.get_current_job()),
                version=getattr(worker, "version", ""),
                python_version=getattr(worker, "python_version", ""),
            )
            for worker in Worker.all()
        ),
        key=lambda w: (w["state"], w["queues"], w["name"]),
    )
    return dict(workers=workers)


@blueprint.context_processor
def inject_interval():
    interval = current_app.config.get("RQ_DASHBOARD_POLL_INTERVAL", 2500)
    return dict(poll_interval=interval)


import importlib
import logging
import os
import sys
from urllib.parse import quote as urlquote, urlunparse

import click
from flask import Flask, Response, request

from . import default_settings
from .version import VERSION
from .web import blueprint, setup_rq_connection


def add_basic_auth(blueprint, username, password, realm="RQ Dashboard"):
    """Add HTTP Basic Auth to a blueprint.

    Note this is only for casual use!

    """

    @blueprint.before_request
    def basic_http_auth(*args, **kwargs):
        auth = request.authorization
        if auth is None or auth.password != password or auth.username != username:
            return Response(
                "Please login",
                401,
                {"WWW-Authenticate": 'Basic realm="{}"'.format(realm)},
            )


def make_flask_app(config, username, password, url_prefix, compatibility_mode=True):
    """Return Flask app with default configuration and registered blueprint."""
    app = Flask(__name__)

    # Start configuration with our built in defaults.
    app.config.from_object(default_settings)

    # Override with any settings in config file, if given.
    if config:
        app.config.from_object(importlib.import_module(config))

    # Override from a configuration file in the env variable, if present.
    if "RQ_DASHBOARD_SETTINGS" in os.environ:
        app.config.from_envvar("RQ_DASHBOARD_SETTINGS")

    # Optionally add basic auth to blueprint and register with app.
    if username:
        add_basic_auth(blueprint, username, password)
    app.register_blueprint(blueprint, url_prefix=url_prefix)

    return app


@click.command()
@click.option(
    "-b",
    "--bind",
    default="0.0.0.0",
    help="IP or hostname on which to bind HTTP server",
)
@click.option(
    "-p", "--port", default=9181, type=int, help="Port on which to bind HTTP server"
)
@click.option(
    "--url-prefix", default="", help="URL prefix e.g. for use behind a reverse proxy"
)
@click.option(
    "--username", default=None, help="HTTP Basic Auth username (not used if not set)"
)
@click.option("--password", default=None, help="HTTP Basic Auth password")
@click.option(
    "-c",
    "--config",
    default=None,
    help="Configuration file (Python module on search path)",
)
@click.option(
    "-H",
    "--redis-host",
    default=None,
    hidden=True,
    help="[DEPRECATED] IP address or hostname of Redis server. Use --redis-url instead",
)
@click.option(
    "-P",
    "--redis-port",
    default=None,
    type=int,
    hidden=True,
    help="[DEPRECATED] Port of Redis server. Use --redis-url instead",
)
@click.option(
    "--redis-password",
    default=None,
    hidden=True,
    help="[DEPRECATED] Password for Redis server. Use --redis-url instead",
)
@click.option(
    "-D",
    "--redis-database",
    default=None,
    type=int,
    hidden=True,
    help="[DEPRECATED] Database of Redis server, Use --redis-url instead",
)
@click.option(
    "-u",
    "--redis-url",
    default=[],
    multiple=True,
    help="Redis URL. Can be specified multiple times. Default: redis://127.0.0.1:6379",
)
@click.option(
    "--redis-sentinels",
    default=None,
    hidden=True,
    help="[DEPRECATED] List of redis sentinels. Use --redis-url instead",
)
@click.option(
    "--redis-master-name",
    default=None,
    hidden=True,
    help="[DEPRECATED] Name of redis master. Only needed when using sentinels. Use --redis-url instead",
)
@click.option(
    "--poll-interval",
    "--interval",
    "poll_interval",
    default=None,
    type=int,
    help="Refresh interval in ms",
)
@click.option(
    "--extra-path",
    default=["."],
    multiple=True,
    help="Append specified directories to sys.path",
)
@click.option(
    "--web-background",
    default=None,
    hidden=True,
    help="[DEPRECATED] Background of the web interface",
)
@click.option(
    "--delete-jobs",
    default=None,
    hidden=True,
    help="[DEPRECATED] Delete jobs instead of cancel",
)
@click.option("--debug/--normal", default=False, help="Enter DEBUG mode")
@click.option(
    "-v", "--verbose", is_flag=True, default=False, help="Enable verbose logging"
)
def run(
    bind,
    port,
    url_prefix,
    username,
    password,
    config,
    redis_host,
    redis_port,
    redis_password,
    redis_database,
    redis_url,
    redis_sentinels,
    redis_master_name,
    poll_interval,
    extra_path,
    web_background,
    debug,
    delete_jobs,
    verbose,
):
    """Run the RQ Dashboard Flask server.

    All configuration can be set on the command line or through environment
    variables of the form RQ_DASHBOARD_*. For example RQ_DASHBOARD_USERNAME.

    A subset of the configuration (the configuration parameters used by the
    underlying flask blueprint) can also be provided in a Python module
    referenced using --config, or with a .cfg file referenced by the
    RQ_DASHBOARD_SETTINGS environment variable.

    """
    if extra_path:
        sys.path += list(extra_path)

    click.echo("RQ Dashboard version {}".format(VERSION))
    app = make_flask_app(config, username, password, url_prefix)
    app.config["DEPRECATED_OPTIONS"] = []
    if redis_url:
        app.config["RQ_DASHBOARD_REDIS_URL"] = redis_url
    else:
        app.config["RQ_DASHBOARD_REDIS_URL"] = "redis://127.0.0.1:6379"
    if redis_host:
        app.config["DEPRECATED_OPTIONS"].append("--redis-host")
    if redis_port:
        app.config["DEPRECATED_OPTIONS"].append("--redis-port")
    if redis_password:
        app.config["DEPRECATED_OPTIONS"].append("--redis-password")
    if redis_database:
        app.config["DEPRECATED_OPTIONS"].append("--redis-database")
    if redis_sentinels:
        app.config["DEPRECATED_OPTIONS"].append("--redis-sentinels")
    if redis_master_name:
        app.config["DEPRECATED_OPTIONS"].append("--redis-master-name")
    if web_background:
        app.config["DEPRECATED_OPTIONS"].append("--web-background")
    if delete_jobs is not None:
        app.config["DEPRECATED_OPTIONS"].append("--delete-jobs")
    if poll_interval:
        app.config["RQ_DASHBOARD_POLL_INTERVAL"] = poll_interval
    # Conditionally disable Flask console messages
    # See: https://stackoverflow.com/questions/14888799
    log = logging.getLogger("werkzeug")
    if verbose:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.ERROR)
        log.error(" * Running on {}:{}".format(bind, port))

    if app.config["DEPRECATED_OPTIONS"] and not redis_url:
        # redis+sentinel://[:password@]host:port[,host2:port2,...][/service_name[/db]][?param1=value1[&param2=value=2&...]]
        scheme = "redis+sentinel" if redis_sentinels else "redis"
        if redis_sentinels:
            netloc = redis_sentinels
        else:
            netloc = redis_host or "localhost"
            if redis_port:
                netloc = "%s:%s" % (netloc, redis_port)
        if redis_password:
            netloc = ":" + urlquote(redis_password) + "@" + netloc
        path = ""
        if redis_master_name:
            path += "/%s" % urlquote(redis_master_name)
        if redis_database:
            path += "/%s" % redis_database
        url = urlunparse((scheme, netloc, path, "", "", ""))
        log.error(
            "Use --redis-url=%s configuration option "
            "instead of specifying host, port and other parameters separately",
            url,
        )
        app.config["RQ_DASHBOARD_REDIS_URL"] = url

    setup_rq_connection(app)
    app.run(host=bind, port=port, debug=debug)


def main():
    run(auto_envvar_prefix="RQ_DASHBOARD")